@@ -49,22 +49,26 @@ struct _NetstatusAppletPrivate
   guint           notify_id;
 };
 
-static void     netstatus_applet_instance_init             (NetstatusApplet      *applet,
-							    NetstatusAppletClass *klass);
-static void     netstatus_applet_class_init                (NetstatusAppletClass *klass);
-static void     netstatus_applet_finalize                  (GObject              *object);
-static gboolean netstatus_applet_key_press_event           (GtkWidget            *widget,
-							    GdkEventKey          *event);
-static void     netstatus_applet_orientation_changed       (NetstatusApplet      *applet,
-							    PanelAppletOrient     orient);
-static void     netstatus_applet_display_help              (BonoboUIComponent    *uic,
-							    NetstatusApplet      *applet);
-static void     netstatus_applet_display_about_dialog      (BonoboUIComponent    *uic,
-							    NetstatusApplet      *applet);
-static void     netstatus_applet_display_properties_dialog (BonoboUIComponent    *uic,
-							    NetstatusApplet      *applet);
-static void     netstatus_applet_display_status_dialog     (NetstatusApplet      *applet);
-static void     netstatus_applet_iface_name_changed        (NetstatusApplet      *applet);
+static void     netstatus_applet_instance_init             (NetstatusApplet           *applet,
+							    NetstatusAppletClass      *klass);
+static void     netstatus_applet_class_init                (NetstatusAppletClass      *klass);
+static void     netstatus_applet_finalize                  (GObject                   *object);
+static gboolean netstatus_applet_key_press_event           (GtkWidget                 *widget,
+							    GdkEventKey               *event);
+static void     netstatus_applet_orientation_changed       (NetstatusApplet           *applet,
+							    PanelAppletOrient          orient);
+static void     netstatus_applet_background_changed        (NetstatusApplet           *applet,
+							    PanelAppletBackgroundType  type,
+							    GdkColor                  *colour,
+							    GdkPixmap                 *pixmap);
+static void     netstatus_applet_display_help              (BonoboUIComponent         *uic,
+							    NetstatusApplet           *applet);
+static void     netstatus_applet_display_about_dialog      (BonoboUIComponent         *uic,
+							    NetstatusApplet           *applet);
+static void     netstatus_applet_display_properties_dialog (BonoboUIComponent         *uic,
+							    NetstatusApplet           *applet);
+static void     netstatus_applet_display_status_dialog     (NetstatusApplet           *applet);
+static void     netstatus_applet_iface_name_changed        (NetstatusApplet           *applet);
 
 static GObjectClass *parent_class;
 
@@ -110,6 +114,8 @@ static void
 netstatus_applet_instance_init (NetstatusApplet      *applet,
 				NetstatusAppletClass *klass)
 {
+  AtkObject *obj;
+
   applet->priv = g_new0 (NetstatusAppletPrivate, 1);
 
   applet->priv->client = gconf_client_get_default ();
@@ -125,6 +131,9 @@ netstatus_applet_instance_init (NetstatusApplet      *applet,
 			    G_CALLBACK (netstatus_applet_display_status_dialog),
 			    applet);
 
+  obj = gtk_widget_get_accessible (GTK_WIDGET (applet));
+  atk_object_set_name (obj, _("Network Monitor"));
+
   gtk_container_add (GTK_CONTAINER (applet), applet->priv->icon);
   gtk_widget_show (applet->priv->icon);
 
@@ -132,6 +141,8 @@ netstatus_applet_instance_init (NetstatusApplet      *applet,
 
   g_signal_connect (applet, "change-orient",
 		    G_CALLBACK (netstatus_applet_orientation_changed), NULL);
+  g_signal_connect (applet, "change_background",
+		    G_CALLBACK (netstatus_applet_background_changed), NULL);
 }
 
 static void
@@ -195,8 +206,10 @@ netstatus_applet_key_press_event (GtkWidget   *widget,
       netstatus_icon_invoke (NETSTATUS_ICON (applet->priv->icon));
       return TRUE;
     default:
-      return FALSE;
+      break;
     }
+
+  return GTK_WIDGET_CLASS (parent_class)->key_press_event (widget, event);
 }
 
 static void
@@ -312,6 +325,55 @@ netstatus_applet_display_about_dialog (BonoboUIComponent *uic,
   gtk_widget_show (applet->priv->about_dialog);
 }
 
+static void
+netstatus_applet_background_changed (NetstatusApplet		*applet,
+				     PanelAppletBackgroundType	 type,
+				     GdkColor			*colour,
+				     GdkPixmap			*pixmap)
+{
+	GtkRcStyle *rc_style;
+
+	gtk_widget_set_style (GTK_WIDGET (applet), NULL);
+	gtk_widget_set_style (GTK_WIDGET (applet->priv->icon), NULL);
+
+	rc_style = gtk_rc_style_new ();
+	gtk_widget_modify_style (GTK_WIDGET (applet), rc_style);
+	gtk_widget_modify_style (GTK_WIDGET (applet->priv->icon), rc_style);
+	g_object_unref (rc_style);
+
+	switch (type) {
+	case PANEL_NO_BACKGROUND:
+		break;
+
+	case PANEL_COLOR_BACKGROUND:
+		gtk_widget_modify_bg (GTK_WIDGET (applet),
+				      GTK_STATE_NORMAL,
+				      colour);
+		gtk_widget_modify_bg (GTK_WIDGET (applet->priv->icon),
+				      GTK_STATE_NORMAL,
+				      colour);
+		break;
+
+	case PANEL_PIXMAP_BACKGROUND: {
+		GtkStyle *style;
+
+		style = gtk_style_copy (GTK_WIDGET (applet)->style);
+
+		if (style->bg_pixmap [GTK_STATE_NORMAL])
+			g_object_unref (style->bg_pixmap [GTK_STATE_NORMAL]);
+		style->bg_pixmap [GTK_STATE_NORMAL] = g_object_ref (pixmap);
+
+		gtk_widget_set_style (GTK_WIDGET (applet), style);
+		gtk_widget_set_style (GTK_WIDGET (applet->priv->icon), style);
+		break;
+	}
+
+	default:
+		g_assert_not_reached();
+		break;
+	}
+}
+
 static void
 netstatus_applet_orientation_changed (NetstatusApplet   *applet,
 				      PanelAppletOrient  orient)
